"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildAppConfig = exports.buildApp = void 0;
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const fs_extra_1 = __importDefault(require("fs-extra"));
const chalk_1 = __importDefault(require("chalk"));
const common_1 = require("../../common");
const path_1 = __importDefault(require("path"));
const cals_1 = require("@cloudbase/cals");
const glob_1 = require("glob");
const DIST_PATH = './dist';
const DEFAULT_INPUTS = {
    config: 'input.json',
    debug: false,
    runtime: process.env.CLOUDBASE_CIID ? lowcode_builder_1.RUNTIME.CI : lowcode_builder_1.RUNTIME.NONE,
    appId: 'test',
    buildTypeList: [lowcode_builder_1.BuildType.WEB],
    generateMpType: 'app',
    subAppSerializeDataList: [],
    dependencies: [],
    extraData: { isComposite: false, compProps: {} },
    mpAppId: '',
    deployOptions: {
        mode: lowcode_builder_1.DEPLOY_MODE.PREVIEW,
        publicPath: '',
    },
    ignoreInstall: false,
    clean: false,
    out: DIST_PATH,
};
class Builder {
    constructor(api, inputs) {
        this.api = api;
        this.logger = api.logger || common_1.Logger;
    }
    async init(inputs) {
        var _a, _b;
        const api = this.api;
        let params = {};
        if (inputs.config === 'local') {
            const [dependencies = [], mainAppSerializeData, subAppSerializeDataList = []] = await Promise.all([
                fs_extra_1.default.readJSON(path_1.default.resolve(this.api.projectPath, 'dependencies.json'), 'utf8'),
                (0, common_1.fileToCode)(this.api.projectPath).then((codeList) => (0, cals_1.codeToCals)(codeList, 'v0')),
                new Promise((resolve) => {
                    (0, glob_1.glob)(path_1.default.resolve(this.api.projectPath, 'src', 'packages', '*'), (err, matches) => {
                        resolve(matches);
                    });
                }).then((list = []) => Promise.all(list.map(async (subPaths) => {
                    const codeList = await (0, common_1.fileToCode)(subPaths);
                    return (0, cals_1.codeToCals)(codeList, 'v0');
                }))),
            ]);
            params = {
                appId: mainAppSerializeData.id,
                mainAppSerializeData,
                subAppSerializeDataList,
                dependencies,
            };
            if ((_a = mainAppSerializeData === null || mainAppSerializeData === void 0 ? void 0 : mainAppSerializeData.extra) === null || _a === void 0 ? void 0 : _a.endpointType) {
                params['runtimeEndpointType'] = (_b = mainAppSerializeData === null || mainAppSerializeData === void 0 ? void 0 : mainAppSerializeData.extra) === null || _b === void 0 ? void 0 : _b.endpointType;
            }
        }
        else {
            this.inputJSONPath = path_1.default.resolve(api.projectPath, inputs.config || DEFAULT_INPUTS.config);
            params = fs_extra_1.default.existsSync(this.inputJSONPath) ? fs_extra_1.default.readJsonSync(this.inputJSONPath) : {};
        }
        this.resolvedInputs = this.normalizInputs(api, resolveInputs(inputs, resolveInputs(params, DEFAULT_INPUTS)));
        if (fs_extra_1.default.existsSync(this.inputJSONPath)) {
            fs_extra_1.default.writeFileSync(this.inputJSONPath, JSON.stringify(this.resolvedInputs, undefined, 2));
        }
    }
    normalizInputs(api, inputs) {
        if (!inputs.mainAppSerializeData) {
            throw new Error('缺少必须参数: mainAppSerializeData');
        }
        // 副作用方法，修改原值
        return (0, lowcode_builder_1.normalizeInputs)(inputs, { envId: api.envId });
    }
    async build(data) {
        var _a;
        this.logger.info('👉 开始生成文件...');
        const outputPath = path_1.default.resolve(this.api.projectPath, this.resolvedInputs.out);
        if (this.resolvedInputs.clean) {
            this.logger.info(`👉 清空输出目录: ${outputPath}`);
            fs_extra_1.default.emptyDirSync(outputPath);
        }
        if (!data) {
            let { appId, mpAppId, mainAppSerializeData, subAppSerializeDataList, dependencies, runtime, buildTypeList, generateMpType, deployOptions, extraData = { isComposite: false, compProps: {} }, ignoreInstall, cdnEndpoints, runtimeEndpointType, } = this.resolvedInputs;
            data = {
                cals: mainAppSerializeData,
                subAppCalsList: subAppSerializeDataList,
                dependencies,
                appKey: appId,
                runtime,
                ignoreInstall,
                buildTypeList,
                mode: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === lowcode_builder_1.DEPLOY_MODE.UPLOAD ? lowcode_builder_1.WebpackModeType.PRODUCTION : lowcode_builder_1.WebpackModeType.NONE,
                deployOptions,
                generateMpType,
                // isCleanDistDir: false,
                plugins: [],
                extraData,
                resourceAppId: mpAppId,
                domain: (_a = mainAppSerializeData.extra) === null || _a === void 0 ? void 0 : _a.domain,
                output: { path: outputPath },
                cdnEndpoints: cdnEndpoints,
                isPrivateMode: !!this.api.privateSettings,
                endpointType: runtimeEndpointType,
            };
        }
        const appPath = await (0, lowcode_builder_1.buildWedaApp)(data);
        this.logger.info(`👉 生成到目录成功：${chalk_1.default.bold.yellow(appPath)}`);
        return { output: outputPath, app: appPath };
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
async function buildApp(ctx, params) {
    var _a, _b, _c, _d;
    const builder = new Builder(ctx, params);
    try {
        await builder.init(params);
        const { output, app: appDir } = await builder.build();
        let inputs;
        if (builder.inputJSONPath && builder.inputJSONPath !== 'local') {
            inputs = fs_extra_1.default.readJsonSync(builder.inputJSONPath);
        }
        const previewPath = (0, lowcode_builder_1.buildAsWebByBuildType)(builder.resolvedInputs.buildTypeList)
            ? path_1.default.join(appDir, 'preview')
            : appDir;
        let cloudbaseRc = {
            version: '2.0',
            envId: '{{env.ENV_ID}}',
            lowcode: {
                'publish:app': {
                    inputs: {
                        appId: builder.resolvedInputs.appId,
                        buildTypeList: builder.resolvedInputs.buildTypeList,
                        projectPath: previewPath,
                        mpAppId: builder.resolvedInputs.mpAppId || undefined,
                        // @ts-expect-error
                        credential: (inputs === null || inputs === void 0 ? void 0 : inputs.credential) || undefined,
                        // @ts-expect-error
                        storage: inputs === null || inputs === void 0 ? void 0 : inputs.storage,
                        // @ts-expect-error
                        webOptions: inputs === null || inputs === void 0 ? void 0 : inputs.webOptions,
                        deployOptions: {
                            // @ts-expect-error
                            mpDeployPrivateKey: (inputs === null || inputs === void 0 ? void 0 : inputs.mpDeployPrivateKey) || undefined,
                            ...builder.resolvedInputs.deployOptions,
                        },
                    },
                },
            },
        };
        const currentCloudRcPath = path_1.default.join(ctx.projectPath, 'cloudbaserc.json');
        const outputCloudRcPath = path_1.default.join(output, 'cloudbaserc.json');
        const currentEnvPath = path_1.default.join(ctx.projectPath, '.env');
        if (fs_extra_1.default.existsSync(currentCloudRcPath)) {
            const current = await fs_extra_1.default.readJSON(currentCloudRcPath);
            cloudbaseRc.envId = current.envId;
            if ((_a = current === null || current === void 0 ? void 0 : current.lowcode) === null || _a === void 0 ? void 0 : _a.privateSettings) {
                cloudbaseRc.lowcode.privateSettings = current.lowcode.privateSettings;
            }
            // 如果 current 存在，则合并 current 的配置
            cloudbaseRc.lowcode['publish:app'] = {
                ...(((_b = current === null || current === void 0 ? void 0 : current.lowcode) === null || _b === void 0 ? void 0 : _b['publish:app']) || {}),
                inputs: {
                    ...cloudbaseRc.lowcode['publish:app'].inputs,
                    ...(((_d = (_c = current === null || current === void 0 ? void 0 : current.lowcode) === null || _c === void 0 ? void 0 : _c['publish:app']) === null || _d === void 0 ? void 0 : _d.inputs) || {}),
                },
            };
            // 如果是原地更新，则进行合并
            if (path_1.default.resolve(currentCloudRcPath) === path_1.default.resolve(outputCloudRcPath)) {
                cloudbaseRc = {
                    ...cloudbaseRc,
                    ...current,
                    lowcode: {
                        ...current.lowcode,
                        'publish:app': cloudbaseRc.lowcode['publish:app'],
                    },
                };
            }
        }
        await fs_extra_1.default.writeFile(outputCloudRcPath, JSON.stringify(cloudbaseRc, undefined, 2));
        if (fs_extra_1.default.existsSync(currentEnvPath)) {
            fs_extra_1.default.copyFileSync(currentEnvPath, path_1.default.join(output, '.env'));
        }
        return { src: appDir, preivew: previewPath };
    }
    catch (e) {
        builder.logger.error(e);
        process.exit(-1);
    }
}
exports.buildApp = buildApp;
async function buildAppConfig(ctx, params) {
    const logger = ctx.logger || common_1.Logger;
    const { runtimeEndpointType, out, domain, buildTypeList } = params;
    try {
        const outPath = out || ctx.projectPath;
        await (0, lowcode_builder_1.buildWedaConfig)({
            output: { path: outPath },
            domain,
            isPrivateMode: !!ctx.privateSettings,
            endpointType: runtimeEndpointType,
            buildTypeList,
        });
        logger.info(`generate success in projectPath: ${outPath}`);
    }
    catch (e) {
        console.log(e);
        logger.error(e);
        process.exit(e);
    }
}
exports.buildAppConfig = buildAppConfig;
