"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deployApp = void 0;
const lowcode_deployer_1 = require("@cloudbase/lowcode-deployer");
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const toolbox_1 = require("@cloudbase/toolbox");
const cloud_api_1 = require("@cloudbase/cloud-api");
const fs_extra_1 = __importDefault(require("fs-extra"));
const common_1 = require("../../common");
const path_1 = __importDefault(require("path"));
const MINIPROGRAM_SETTING = {
    es6: true,
    es7: true,
    minify: true,
    codeProtect: false,
    useCompilerPlugins: ['typescript'],
};
const DEFAULT_INPUTS = {
    _inputFile: 'input.json',
    appId: 'test',
    mpAppId: '',
    buildTypeList: [lowcode_builder_1.BuildType.WEB],
    deployOptions: {
        mode: (process.env.deployMode || lowcode_builder_1.DEPLOY_MODE.PREVIEW),
    },
};
class Deoloyer {
    constructor(api, inputs) {
        var _a, _b, _c, _d;
        this.api = api;
        this.logger = api.logger || common_1.Logger;
        let inputJSONPath = path_1.default.resolve(api.projectPath, inputs._inputFile || DEFAULT_INPUTS._inputFile);
        let params = fs_extra_1.default.existsSync(inputJSONPath) ? fs_extra_1.default.readJsonSync(inputJSONPath) : {};
        this._resolvedInputs = this.normalizInputs(api, resolveInputs(inputs, resolveInputs(params, DEFAULT_INPUTS)));
        (0, common_1.initLowcodeCloudApiEnv)({ privateSettings: this.api.privateSettings });
        if (this.api.privateSettings) {
            if ((_a = this.api.privateSettings.credential) === null || _a === void 0 ? void 0 : _a.weda) {
                const accesstokenConfig = (_b = this.api.privateSettings.credential) === null || _b === void 0 ? void 0 : _b.weda;
                this._useAccessToken = true;
                const e = this._resolvedInputs.envId;
                this.cloudService = new lowcode_deployer_1.TCBAPICloudApiService({
                    service: 'lowcode',
                    version: '2021-01-08',
                    timeout: 10000,
                    // env: this.resolvedInputs.envId,
                    // apiOrigin: ``,
                    // endpoint: process.env.PRIVATE_ENDPOINT || `http://${this.resolvedInputs?.webOptions?.domain}`,
                    env: e,
                    apiOrigin: `${e}.ap-shanghai.tcb-api.tencentcloudapi.com`,
                    endpoint: `https://pre.ap-shanghai.tcb-api.tencentcloudapi.com`,
                    username: accesstokenConfig === null || accesstokenConfig === void 0 ? void 0 : accesstokenConfig.username,
                    password: accesstokenConfig === null || accesstokenConfig === void 0 ? void 0 : accesstokenConfig.password,
                    localstorage: new lowcode_deployer_1.LocalStorage(this.api.projectPath),
                });
            }
            else {
                this.cloudService = cloud_api_1.CloudApiService.getInstance({
                    service: 'lowcode',
                    proxy: (0, toolbox_1.getProxy)(),
                    credential: this.api.privateSettings.credential,
                });
                const originalRequest = this.cloudService.request;
                this.cloudService.request = (actionOrOptions, data) => {
                    if (typeof actionOrOptions === 'string') {
                        return originalRequest.call(this.cloudService, actionOrOptions, {
                            privateUin: this.api.privateSettings.privateUin,
                            ...data,
                        });
                    }
                    else {
                        const newData = {
                            privateUin: this.api.privateSettings.privateUin,
                            ...actionOrOptions.data,
                        };
                        return originalRequest.call(this.cloudService, {
                            ...actionOrOptions,
                            data: newData,
                        });
                    }
                };
            }
        }
        else {
            // this.cloudService = CloudApiService.getInstance('lowcode');
        }
        const { appId, mpAppId, envId, buildTypeList, deployOptions, projectPath, webOptions = {} } = this._resolvedInputs;
        const type = (0, lowcode_builder_1.buildAsWebByBuildType)(buildTypeList) ? 'WEB' : 'MP';
        const mode = deployOptions.mode === lowcode_builder_1.DEPLOY_MODE.UPLOAD ? lowcode_deployer_1.EDeployMode.PRODUCTION : lowcode_deployer_1.EDeployMode.PREVIEW;
        const deployerOptions = {
            logger: this.logger,
            type,
            mode,
            appId,
            envId,
            projectPath: projectPath,
            ignores: [],
            mpAppId,
            credential: this._resolvedInputs.credential || this.api.privateSettings.credential,
            // @ts-expect-error
            artifactOptions: process.env.CLOUDBASE_CIID
                ? {
                    ...this._resolvedInputs.storage,
                    baseKey: `lca/${appId}${process.env.CLOUDBASE_CIID ? `/${process.env.CLOUDBASE_CIID}` : ''}`,
                }
                : undefined,
        };
        if (type === 'MP') {
            let projectJson = fs_extra_1.default.readJsonSync(path_1.default.resolve(projectPath, 'project.config.json'));
            let { cloudfunctionRoot, miniprogramRoot = './' } = projectJson;
            deployerOptions.mode =
                type === 'MP' && deployOptions.mpAppId !== deployOptions.targetMpAppId
                    ? lowcode_deployer_1.EDeployMode.PRODUCTION
                    : deployerOptions.mode;
            deployerOptions.ignores = ['node_modules/**/*'].concat(cloudfunctionRoot ? [path_1.default.join(cloudfunctionRoot, '**/*')] : []);
            deployerOptions.miniprogramOptions = {
                privateKeyPath: path_1.default.join(projectPath, `./private.${deployOptions.mpAppId}.key`),
                uploadOptions: {
                    version: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.version) || '1.0.0',
                    desc: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.description) || '',
                    setting: MINIPROGRAM_SETTING,
                },
                previewOptions: {
                    // @ts-expect-error
                    qrcodeOutputPath: path_1.default.resolve(projectPath, './qrcode.jpg'),
                    // @ts-expect-error
                    pagePath: (_d = (_c = fs_extra_1.default.readJsonSync(path_1.default.resolve(projectPath, miniprogramRoot, 'app.json'))) === null || _c === void 0 ? void 0 : _c.pages) === null || _d === void 0 ? void 0 : _d[0],
                    setting: MINIPROGRAM_SETTING,
                },
            };
        }
        else {
            deployerOptions.webOptions = {
                buildTypeList,
                ...webOptions,
            };
        }
        this.instance = new lowcode_deployer_1.deployer(deployerOptions);
    }
    normalizInputs(api, inputs) {
        var _a, _b, _c, _d;
        /**
         * mainAppSerializeData  为了兼容 lowcode plugin 参数
         */
        inputs.envId = ((_b = (_a = inputs.mainAppSerializeData) === null || _a === void 0 ? void 0 : _a.extra) === null || _b === void 0 ? void 0 : _b.envid) || inputs.envId || api.envId;
        inputs.projectPath = inputs.projectPath || api.projectPath;
        inputs.webOptions = inputs.webOptions || {};
        if (!(0, lowcode_builder_1.buildAsWebByBuildType)(inputs.buildTypeList)) {
            if (!((_c = inputs.deployOptions) === null || _c === void 0 ? void 0 : _c.mpDeployPrivateKey)) {
                if (inputs.mpDeployPrivateKey) {
                    inputs.deployOptions.mpDeployPrivateKey = inputs.mpDeployPrivateKey;
                }
                else if (process.env.mpDeployPrivateKey) {
                    inputs.deployOptions.mpDeployPrivateKey = process.env.mpDeployPrivateKey;
                }
            }
            if (!((_d = inputs.deployOptions) === null || _d === void 0 ? void 0 : _d.mpAppId)) {
                inputs.deployOptions.mpAppId = inputs.mpAppId;
            }
            if (inputs.deployOptions.mpDeployPrivateKey) {
                this._generateMPDeployPrivateKey = true;
                fs_extra_1.default.writeFileSync(path_1.default.join(inputs.projectPath, `./private.${inputs.deployOptions.mpAppId}.key`), inputs.deployOptions.mpDeployPrivateKey, 'base64');
            }
        }
        return inputs;
    }
    async deploy() {
        try {
            this.logger.info('👉 开始部署产物...');
            const res = await this.instance.deploy();
            this.logger.info('👉 部署成功');
            return res;
        }
        catch (e) {
            throw e;
        }
        finally {
            fs_extra_1.default.removeSync(
            // @ts-expect-error
            path_1.default.join(this._resolvedInputs.projectPath, `./private.${this._resolvedInputs.deployOptions.mpAppId}.key`));
        }
    }
    async syncCloud({ link }) {
        if (this._resolvedInputs.syncCloud && this.cloudService) {
            const ciId = process.env.CLOUDBASE_CIID;
            if (!ciId) {
                throw new Error('上传更新云端部署记录失败，缺乏必须构建唯一ID ciId');
            }
            const { appId, buildTypeList, mpAppId, deployOptions } = this._resolvedInputs;
            const params = {
                appId,
                ciId,
                version: deployOptions.version,
                type: buildTypeList[0],
                status: 'success',
                mode: deployOptions.mode,
                description: deployOptions.description,
                appHistoryId: deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.appHistoryId,
                spMode: 0,
                wxAppId: mpAppId,
                preview: {
                    web: link,
                },
            };
            if (this._useAccessToken) {
                await this.cloudService.request('InvokeLcapServerFunc', {
                    MethodType: 'POST',
                    MethodPath: 'cliapi',
                    RequestParam: JSON.stringify({
                        action: await this.getApiName('CreateDeployHistory'),
                        ...params,
                    }),
                });
            }
            else {
                await this.cloudService.request(await this.getApiName('CreateDeployHistory'), params);
            }
        }
    }
    async getApiName(publicApiName) {
        var _a;
        try {
            if (!this._privateApiMap) {
                this._privateApiMap = await fs_extra_1.default.readJSON(path_1.default.join(toolbox_1.cloudbaseConfigDir, 'lowcode-private-api.json'), {
                    encoding: 'utf8',
                });
            }
            return (_a = this._privateApiMap[publicApiName]) !== null && _a !== void 0 ? _a : publicApiName;
        }
        catch (_) {
            return publicApiName;
        }
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
async function deployApp(ctx, params) {
    try {
        const instance = new Deoloyer(ctx, params);
        const res = await instance.deploy();
        await instance.syncCloud({ link: res.link });
    }
    catch (e) {
        console.error(e);
        process.exit(-1);
    }
}
exports.deployApp = deployApp;
