"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.graceDebugComps = void 0;
const webpack_1 = __importDefault(require("webpack"));
const vm = __importStar(require("vm"));
const lodash_1 = require("lodash");
const chokidar_1 = require("chokidar");
const graceBuild_1 = require("../graceBuild");
const common_1 = require("../../common");
const app_1 = require("../../app");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const toolbox_1 = require("@cloudbase/toolbox");
const webpack_2 = require("../webpack");
const gracePublish_1 = require("../gracePublish");
const semver = __importStar(require("semver"));
const graceBuild_2 = require("../graceBuild");
// const DebugBaseUrl = 'https://console.cloud.tencent.com/lowcode/app/editor'
const DebugBaseUrl = 'https://lcap.cloud.tencent.com/v3/dev.html';
class GraceDebugService extends graceBuild_1.GraceBuildService {
    constructor(info) {
        var _a;
        super(info);
        this.debugInfo = info;
        // this.debugPage = `${DebugBaseUrl}?id=test&debugPort=${info.debugPort}&libName=${info.name}&libVersion=beta`;
        this.debugPage = `${(_a = this.debugInfo.debugBaseUrl) !== null && _a !== void 0 ? _a : DebugBaseUrl}?appId=test&mainAppId=test&&detailAppId=tes&source=null&debugPort=${info.debugPort}&libName=${info.name}&libVersion=beta`;
        this.debugAssetsPath = './.temp/assets';
        (0, common_1.initLowcodeCloudApiEnv)(info);
    }
    async debug() {
        // 清理Dist
        await this.clean();
        // 自动识别
        await this.recognition();
        // 启动DebugServer
        const koaPort = await this.runKoaDevServer();
        const config = this.createWebpackConfig();
        await this.runWebpackDevServer(config, koaPort);
    }
    async runKoaDevServer() {
        const { wxDevtoolPath, platformRoot, context, name } = this.debugInfo;
        // 创建Server
        const koaPort = await (0, toolbox_1.getPort)();
        const koaServer = new common_1.KoaServer({
            port: koaPort,
            staticPath: path_1.default.resolve(this.info.context, this.debugAssetsPath),
        });
        // 绑定预览服务
        if (platformRoot.mp && fs_extra_1.default.existsSync(path_1.default.resolve(context, this.getPlatformEntry(platformRoot.mp)))) {
            const previewService = new app_1.PreviewService({
                port: koaPort,
                wxDevtoolPath,
                mode: 'debug',
            });
            previewService.setLocalCompsPath(name, path_1.default.resolve(context, this.getPlatformEntry(platformRoot.mp)), path_1.default.resolve(context, this.debugAssetsPath));
            (0, app_1.bindRouter)(koaServer, previewService);
        }
        // 启动Server
        koaServer.listen();
        return koaPort;
    }
    async runWebpackDevServer(config, koaPort) {
        var _a;
        this.info.logger.info('组件库 - 启动调试服务器\n');
        const { debugPort } = this.debugInfo;
        const debugHost = '127.0.0.1';
        const gracePublishService = new gracePublish_1.GracePublishService(Object.assign(this.debugInfo, { isAdmin: false }));
        // 初始化参数
        const devServerOptions = {
            hot: 'only',
            host: debugHost,
            port: debugPort,
            static: {
                serveIndex: true,
                publicPath: `http://${debugHost}:${debugPort}/static/`,
            },
            headers: {
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Allow-Headers': '*',
            },
            allowedHosts: 'all',
            proxy: {
                '/api': `http://${debugHost}:${koaPort}`,
                '/assets': `http://${debugHost}:${koaPort}`,
            },
            compress: true,
        };
        /**
         * 这里的 entry 有 control 和 main，分开的原因如下：
         * 1. 减少不必要的 hot-update，按需 hot-update
         * 2. 不同的 hotUpdateGlobal 名称避免同名在编辑区后加载的覆盖先加载的，导致不稳定的编辑区热更失败
         */
        const configs = Object.keys(config.entry).map((entryKey) => {
            var _a;
            return {
                ...config,
                entry: {
                    [entryKey]: (_a = config.entry) === null || _a === void 0 ? void 0 : _a[entryKey],
                },
                output: {
                    ...config.output,
                    hotUpdateGlobal: `webpackHotUpdate_weapps_materials_${this.debugInfo.name}_${entryKey}`,
                },
                stats: 'errors-only',
                cache: false,
            };
        });
        // 初始化实例
        const devServer = new webpack_dev_server_1.default(devServerOptions, (0, webpack_1.default)(configs));
        // 开启监听
        await devServer.start();
        common_1.Logger.info('\n进入页面编辑器，左上角点击【···】，开启【自定义组件开发】模式，即可预览本地的组件。\n');
        // 额外通过 websocket 发送自定义的信息
        const sockjsServer = (_a = devServer.webSocketServer) === null || _a === void 0 ? void 0 : _a.implementation;
        if (sockjsServer) {
            sockjsServer.on('connection', (client) => {
                devServer.sendMessage([client], 'libName', this.debugInfo.name);
            });
        }
        /**
         * 监听小程序组件代码
         */
        const mpConfig = configs.find((item) => item.entry['control']);
        if (!mpConfig)
            return;
        const configOnlyForControl = (0, lodash_1.cloneDeep)(mpConfig);
        configOnlyForControl.output.filename = 'control.js'; // 避免与上边生成的文件互相覆盖
        // 开dev和sourcemap 会导致内容里面有 window.hotUpdate 相关内容，下面那个node:vm跑不起来
        configOnlyForControl.mode = 'production';
        configOnlyForControl.devtool = false;
        const compilerForControl = (0, webpack_1.default)(configOnlyForControl);
        let lcds;
        const webpackRun = (0, lodash_1.debounce)(() => {
            compilerForControl.run((err) => {
                if (err) {
                    console.error(err);
                }
            });
        }, 300);
        const mpEntry = typeof this.debugInfo.platformRoot.mp === 'object'
            ? this.debugInfo.platformRoot.mp.entry
            : this.debugInfo.platformRoot.mp;
        (0, chokidar_1.watch)(path_1.default.resolve(this.debugInfo.context, mpEntry), { ignoreInitial: true }).on('all', () => {
            webpackRun();
        });
        // 生成 lcds config: meta.json - 这里通过新的 webpack 配置来生成是为了避免 webpack dev server 插入代码的干扰，会导致异常
        compilerForControl.hooks.assetEmitted.tap('load_lcds_config', (file, info) => {
            if (file === configOnlyForControl.output.filename) {
                const content = info.content.toString('utf-8');
                const ctx = { window: {} };
                vm.createContext(ctx);
                vm.runInContext(content, ctx);
                lcds = ctx.window[`@weapps-materials-control-${this.debugInfo.name}`].lcds;
            }
        });
        // 生成的 mp source zip 包可通过 http://127.0.0.1:8388/assets/sourceMp.zip 下载
        compilerForControl.hooks.done.tapPromise('build_mp_zip', async () => {
            var _a;
            const { context, platformRoot, configRoot } = this.debugInfo;
            await gracePublishService.zipSourceCode({
                sourcePath: './.temp/assets/sourceMp',
                sourceZippedFileName: './.temp/assets/sourceMp.zip',
                platform: 'mp',
                platformRoot,
                context,
                configRoot,
                ignoreGlobs: [],
                lcdsConfig: lcds,
            });
            if ((_a = devServer === null || devServer === void 0 ? void 0 : devServer.webSocketServer) === null || _a === void 0 ? void 0 : _a.clients) {
                devServer.sendMessage(devServer.webSocketServer.clients, 'mp_hash', this.debugInfo.name);
            }
        });
        // 初始先生成一遍
        webpackRun();
    }
    createWebpackConfig() {
        var _a;
        const { name, context, experimental = {} } = this.info;
        const isTs = this.isTs(this.info);
        const packageJSON = fs_extra_1.default.readJsonSync(path_1.default.join(context, 'package.json'));
        let vueVersion;
        const vueVersionInPackage = (_a = packageJSON.dependencies) === null || _a === void 0 ? void 0 : _a.vue;
        if (vueVersionInPackage == null) {
            vueVersion = graceBuild_2.VUEVersion.NO;
        }
        else {
            const vueRawVersion = semver.minVersion(vueVersionInPackage);
            vueVersion =
                semver.valid(vueRawVersion) === null
                    ? graceBuild_2.VUEVersion.NO
                    : semver.gt(vueRawVersion, '2.7.16')
                        ? graceBuild_2.VUEVersion.VUE3
                        : graceBuild_2.VUEVersion.VUE2;
        }
        const service = new webpack_2.WebpackService({
            name,
            context,
            entry: {
                control: this.configEntryPath,
                main: this.mainEntryPath,
            },
            output: this.outputPath,
            mode: 'development',
            isTs,
            splitChunks: false,
            publicPath: 'http://127.0.0.1:8388/static/',
            sourceMap: false,
            experimental,
            enableReactRefreshInDev: false,
            configRoot: this.info.configRoot,
            vueVersion,
        });
        return service.createConfig();
    }
}
async function graceDebugComps(info) {
    const service = new GraceDebugService({ ...info, command: 'debug' });
    await service.debug();
}
exports.graceDebugComps = graceDebugComps;
