"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const mime_types_1 = __importDefault(require("mime-types"));
const recast_1 = require("recast");
const ast_types_1 = require("ast-types");
const svgo_1 = require("svgo");
/**
 * 图片转base64 loader
 * @param source
 * @returns
 */
function default_1(source, meta, extra) {
    const resourcePath = this.resourcePath;
    if (!/\.(json|js|ts)$/.test(resourcePath))
        return source;
    const iconProp = ['icon', 'figureIcon'];
    if (typeof source === 'string' && (this.resourcePath.endsWith('.ts') || this.resourcePath.endsWith('js'))) {
        try {
            const tsAst = (0, recast_1.parse)(source, {
                parser: require('recast/parsers/typescript'),
            });
            const context = this.context;
            (0, ast_types_1.visit)(tsAst, {
                visitObjectProperty(nodePath) {
                    var _a, _b, _c, _d, _e;
                    // @ts-expect-error object property
                    if (iconProp.includes((_a = nodePath.node.key) === null || _a === void 0 ? void 0 : _a.name) && ((_e = (_d = (_c = (_b = nodePath.parent) === null || _b === void 0 ? void 0 : _b.parent) === null || _c === void 0 ? void 0 : _c.node) === null || _d === void 0 ? void 0 : _d.key) === null || _e === void 0 ? void 0 : _e.name) === 'meta') {
                        // @ts-expect-error
                        const icon = nodePath.node.value.value;
                        const iconPath = path_1.default.resolve(context, icon);
                        if (icon) {
                            if (fs_1.default.existsSync(iconPath)) {
                                // @ts-expect-error
                                nodePath.node.value.value = createDataUri(iconPath);
                                return false;
                            }
                            else if (!isUrlAbsolute(icon)) {
                                console.warn(`Unable to find icon in ${iconPath}`);
                            }
                        }
                    }
                    this.traverse(nodePath);
                },
            });
            return (0, recast_1.print)(tsAst).code;
        }
        catch (error) {
            console.error(error);
            // 兜底不处理
            return source;
        }
    }
    const value = typeof source === 'string' ? JSON.parse(source) : source;
    iconProp.forEach((d) => {
        const icon = lodash_1.default.get(value, `meta.${d}`, '');
        const iconUrl = path_1.default.resolve(this.context, icon);
        if (icon) {
            if (fs_1.default.existsSync(iconUrl)) {
                lodash_1.default.set(value, `meta.${d}`, createDataUri(iconUrl));
            }
            else if (!isUrlAbsolute(icon)) {
                console.warn(`Unable to find ${d} in ${iconUrl}`);
            }
        }
    });
    return JSON.stringify(value)
        .replace(/\u2028/g, '\\u2028')
        .replace(/\u2029/g, '\\u2029');
}
exports.default = default_1;
function createDataUri(iconPath) {
    const mimeType = mime_types_1.default.lookup(path_1.default.extname(iconPath).replace(/^\./, '')); // 获取图片媒体类型
    if (mimeType === 'image/svg+xml') {
        let svgStr = fs_1.default.readFileSync(iconPath, { encoding: 'utf-8' });
        try {
            const svg = (0, svgo_1.optimize)(svgStr, {
                multipass: true,
                plugins: [
                    {
                        name: 'preset-default',
                        params: {
                            overrides: {
                                removeViewBox: false,
                            },
                        },
                    },
                ],
            });
            svgStr = svg.data;
        }
        catch (error) {
            console.warn(`Failed to optimize ${iconPath}`);
        }
        const dataUri = `data:${mimeType},${encodeURIComponent(svgStr)}`;
        return dataUri;
    }
    const base64str = fs_1.default.readFileSync(iconPath, 'base64');
    return `data:${mimeType};base64,${base64str}`;
}
const isUrlAbsolute = (url) => url.indexOf('://') > 0 || url.indexOf('//') === 0;
